      PROGRAM FNALTEST
C
C---   Produce B mesons with the PYTHIA generator and find the relative frequency
C      of production of various B mesons and baryons. In the inialization
C      routine UPYTHINIT, use PYGIVE to input non-default values
C      of some PYTHIA variables.
C
C---   S.F.TAKACH --- 2/12/90
C

C---   PYTHIA common blocks containing the variables accessible via PYGIVE.
C
C  NOTE: Cannot use "IMPLICIT NONE" with this header
#include "main60.f"

C---   The number of events to generate.
      INTEGER NEVENT

C---   The level of detail to output for end-of-run statistics.
      INTEGER MSTAT
      PARAMETER (MSTAT=1)

C---   The IDs of the particle we want to examine.
      INTEGER NID
      PARAMETER (NID=15)
      INTEGER ID(NID)

C---   Code numbers of (open) b-states, in the order:
C      B0,B0*,B+,B+*,Bs0,Bs0*,Sigma_b-,Sigma_b-*,Lambda_b0,Sigma_b0,Sigma_b0*,
C      Sigma_b+,Sigma_b+*. (See the JETSET72 manual for particle flavor (KF)
C      codes).
      DATA ID/511,513,521,523,531,533,5112,5114,5122,5212,5214,
     1        5222,5224,0,0/

C---   I/O unit numbers
      INTEGER PYIN,PYOUT,PYZEB
      PARAMETER (PYIN=7,PYOUT=6,PYZEB=22)

C---   Some dummy loop counters. 
C      We cannot use K,N in this program because JETSET uses them.
      INTEGER I,J,L,M

C---   The HBOOK common
      REAL HMEMOR
      COMMON/PAWC/HMEMOR(20000)

C---   Initialize I/O and PYTHIA.
      CALL UPYTHINIT(NEVENT)

C---   Generate PYTHIA events. Make histos along the way.
      DO 10 I=1,NEVENT
        IF (MOD(I,100).EQ.0) WRITE(PYOUT,9001) I
 9001   FORMAT(1X,'Event Number: ',3X,I6)
        CALL PYEVNT
C---    List the first event
        IF(I.EQ.1) CALL PYLIST(2)
C---   Loop over the particles produced at the primary vertex.
C      Test K(J,1) to make sure only undecayed particles are examined. 
C      Test K(J,2) against array ID to find the particles we want.
C      Histogram the ID array index, L, every time we find a particle 
C      with an identification number = +/-ID(L)
C---
        DO 15 J=1,N
          IF ((K(J,1).GE.1).AND.(K(J,1).LE.10)) THEN
            DO 20 L=1,NID
              IF (ABS(K(J,2)).EQ.ID(L)) THEN
                CALL HF1(1,FLOAT(L)+0.5,1.0)                
              ENDIF
 20         CONTINUE
          ENDIF
 15     CONTINUE

C---   Continue generating events.
 10   CONTINUE

C---   Print end of run statistics.
      CALL PYSTAT(MSTAT)

C---   Send HBOOK ouput to the user unit.
      CALL HIDOPT(0,'BLAC')
      CALL HPRINT(0)
C---
      CALL HRPUT(0,'FNALTEST.RZ','N')
      
C---   Close up shop
      CLOSE(PYOUT)
      CLOSE(PYIN)

      STOP
      END
C
C
      SUBROUTINE UPYTHINIT(NEVENT)
C
C---   User initialization of the program. 
C      Open files for I/O. 
C      Read PYTHIA initialization variables and set any PYTHIA variables
C      to non-default values. 
C
C      Use the      PYTHIA interface routine PYGIVE to get input variables from 
C      an input file with the following format:
C
C            'Some kind of comment'
C            NEVENT
C            'FRAME','BEAM','TARGET',WIN
C            'PYVAR0=X;PYVAR1(I,J)=Y;PYVAR2(K)=Z'
C            'PYVAR3(L,M)=T;PYVAR4=U'
C            'PYVAR5(L)=V; ...etc.'
C            'etc.....'
C            'etc.....'
C
C      Where:
C          NEVENT is the number of events to generate.
C          FRAME,BEAM,TARGET,WIN = PYTHIA initialization variables.
C          PYVARi refers to some PYTHIA variable in the PYTHIA common blocks.
C          I,J,K,L,M are indices for array variables.
C      For example, MSTU(32) = ...
C      Lines are read as character strings from the input file and
C      interpreted by PYGIVE. "NEVENT" =
C---
      IMPLICIT NONE

C---   The number of events to generate.
      INTEGER NEVENT

C---   PYTHIA initialization variables. See PYTHIA document,
C      subroutine PYINIT for their meaning.
      CHARACTER*4 FRAME
      CHARACTER*5 BEAM,TARGET
      CHARACTER*80 COMMENT,LINE
      double precision WIN

C---   I/O unit numbers
      INTEGER PYIN,PYOUT,PYZEB
      PARAMETER (PYIN=7,PYOUT=6,PYZEB=22)

C---   Name of the user output file
      CHARACTER*50 USROUT

C---   The HBOOK common
      REAL HMEMOR
      COMMON/PAWC/HMEMOR(20000)

C---   Setup the HBOOK memory limit.
      CALL HLIMIT(20000)

C---   Book histograms.
      CALL HBOOK1(1,'UN-NORM PROD FREQ: B STATES$',20,0.0,20.0,0.0)
      
C---   Get the names of the files to be used.
C      Read from the default input stream (usually batch).

C---   Open the input file
      OPEN(PYIN,FILE='fnal_in.dat',STATUS='OLD')

      READ(PYIN,9500) USROUT
 9500 FORMAT(A50)

C---   Open the output file for histos, etc.
C      Write the name of the input file to it.
      OPEN(PYOUT,FILE=USROUT,STATUS='NEW')
      WRITE(PYOUT,9000) 
 9000 FORMAT(1X,'Input File: fnal_in.dat'/)
C---
      CALL HOUTPU(PYOUT)      
      
C---   Read the comment line and the number of events to generate.
C---    Write this information to the output file
      READ(PYIN,*) COMMENT
      READ(PYIN,*) NEVENT
      WRITE(PYOUT,9001) COMMENT
      WRITE(PYOUT,9002) NEVENT
 9001 FORMAT(10X,A80)
 9002 FORMAT(10X,I10,' events will be generated'/)

C---   Read the PYTHIA initialization variables.
      READ(PYIN,*) FRAME,BEAM,TARGET,WIN

C---   Now loop over the remaining character lines in the input file. 
C      Feed each line to PYGIVE and let it interpret the line.
C      When end-of-file is reached, close the input unit.
 100  READ(PYIN,*,END=200) LINE
      CALL PYGIVE(LINE)
      GOTO 100
 200  CLOSE(PYIN)

C---   Initialize PYTHIA
      CALL PYINIT(FRAME,BEAM,TARGET,WIN)

      RETURN
      END
